import { Collection, WithId } from 'mongodb';
import { BaseService, Service } from '../../../core';
import { Entity, EntityServiceSurface } from '../interfaces/entity-manager';
declare type MongoDoc<TValue extends Record<string, any>> = {
    _id?: string;
    created?: number;
    deleted?: number;
} & TValue;
export declare type EntityIndexKey<TValue extends Record<string, any>> = Partial<{
    [K in keyof TValue]: boolean | 1 | -1;
}>;
export interface EntityManagerServiceOptions<TValue extends Record<string, any>> {
    id: string;
    indexes?: {
        key: EntityIndexKey<TValue>;
        unique?: boolean;
    }[];
}
export declare abstract class EntityManagerService<TValue extends Record<string, any>> extends BaseService implements Service<EntityServiceSurface<TValue>> {
    private _module;
    private _collection;
    get collection(): Collection<MongoDoc<TValue>>;
    indexedFields: Set<string>;
    options: EntityManagerServiceOptions<TValue>;
    constructor(options: EntityManagerServiceOptions<TValue>);
    createIndexes(): Promise<void>;
    protected _parseEntity(doc: WithId<MongoDoc<TValue>>): Entity<TValue>;
    get(id: string): Promise<Entity<TValue> | undefined>;
    getByIds(ids: string[]): Promise<(Entity<TValue> | undefined)[]>;
    create(value: TValue): Promise<[newId: string]>;
    delete(id: string): Promise<void>;
}
export {};
//# sourceMappingURL=entity-manager.d.ts.map