import { Collection, ObjectId, WithId } from 'mongodb';
import { BaseService, ServiceInterface } from '../../../core';
declare type MongoDoc = {
    _id: ObjectId;
    createdTime: number;
    deletedTime?: number;
};
export interface Entity {
    id: string;
    createdTime: number;
    deletedTime?: number;
}
export declare type EntityServiceInterface = {
    errors: {
        INVALID_ENTITY: {
            id: string;
        };
    };
    events: {
        CREATED: {
            id: string;
            time: number;
        };
        DELETED: {
            id: string;
            time: number;
        };
    };
};
export declare abstract class EntityService<TServiceInf extends ServiceInterface & EntityServiceInterface = any> extends BaseService<TServiceInf> {
    private _module;
    private _collection;
    get collection(): Collection<MongoDoc>;
    protected _parseEntity(doc: WithId<MongoDoc>): Entity;
    protected _get(id: string): Promise<Entity | undefined>;
    protected _getByIds(ids: string[]): Promise<(Entity | undefined)[]>;
    protected _getAll(): Promise<Entity[]>;
    protected _create(): Promise<[newId: string]>;
    protected _delete(id: string): Promise<void>;
}
export {};
//# sourceMappingURL=entity.d.ts.map